<?xml version="1.0" encoding="utf-8"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                version="1.0">

  <xsl:param name="database"/>

  <xsl:template match="configuration">
    <xsl:copy>
      <xsl:apply-templates select="@*"/>
      <!-- Find out some additional information. -->
      <xsl:variable name="cpu-socket" select="$database/database/base-system/base-system-enum[@value = current()/details/base-system/@value]/@cpu-socket"/>
      <xsl:variable name="interface" select="$database/database/base-system/base-system-enum[@value = current()/details/base-system/@value]/@interface"/>
      <!-- Complete all multiple choice fields. -->
      <details>
        <base-system>
          <xsl:variable name="value" select="details/base-system/@value"/>
          <xsl:attribute name="value"><xsl:value-of select="$value"/></xsl:attribute>
          <xsl:for-each select="$database/database/base-system/base-system-enum">
            <xsl:copy>
              <xsl:apply-templates select="@*"/>
              <xsl:if test="@value = $value">
                <xsl:attribute name="value-is-set">true</xsl:attribute>
              </xsl:if>
            </xsl:copy>
          </xsl:for-each>
        </base-system>
        <cpu>
          <xsl:variable name="value" select="details/cpu/@value"/>
          <xsl:attribute name="value"><xsl:value-of select="$value"/></xsl:attribute>
          <xsl:for-each select="$database/database/cpu/cpu-enum[@cpu-socket = $cpu-socket or not(@cpu-socket)]">
            <xsl:copy>
              <xsl:apply-templates select="@*"/>
              <xsl:if test="@value = $value">
                <xsl:attribute name="value-is-set">true</xsl:attribute>
              </xsl:if>
            </xsl:copy>
          </xsl:for-each>
        </cpu>
      </details>
      <memory>
        <xsl:for-each select="memory/memory-unit">
          <xsl:copy>
            <xsl:variable name="value" select="@value"/>
            <xsl:attribute name="value"><xsl:value-of select="$value"/></xsl:attribute>
            <xsl:for-each select="$database/database/memory-unit/memory-unit-enum">
              <xsl:copy>
                <xsl:apply-templates select="@*"/>
                <xsl:if test="@value = $value">
                  <xsl:attribute name="value-is-set">true</xsl:attribute>
                </xsl:if>
              </xsl:copy>
            </xsl:for-each>
          </xsl:copy>
        </xsl:for-each>
      </memory>
      <hard-disks>
        <xsl:for-each select="hard-disks/hard-disk">
          <xsl:copy>
            <xsl:variable name="value" select="@value"/>
            <xsl:attribute name="value"><xsl:value-of select="$value"/></xsl:attribute>
            <xsl:for-each select="$database/database/hard-disk/hard-disk-enum[@interface = $interface or not(@interface)]">
              <xsl:copy>
                <xsl:apply-templates select="@*"/>
                <xsl:if test="@value = $value">
                  <xsl:attribute name="value-is-set">true</xsl:attribute>
                </xsl:if>
              </xsl:copy>
            </xsl:for-each>
          </xsl:copy>
        </xsl:for-each>
      </hard-disks>
      <storage>
        <xsl:for-each select="storage/storage-unit">
          <xsl:copy>
            <xsl:variable name="value" select="@value"/>
            <xsl:attribute name="value"><xsl:value-of select="$value"/></xsl:attribute>
            <xsl:for-each select="$database/database/storage-unit/storage-unit-enum">
              <xsl:copy>
                <xsl:apply-templates select="@*"/>
                <xsl:if test="@value = $value">
                  <xsl:attribute name="value-is-set">true</xsl:attribute>
                </xsl:if>
              </xsl:copy>
            </xsl:for-each>
          </xsl:copy>
        </xsl:for-each>
      </storage>
      <accessories>
        <xsl:variable name="values" select="accessories/accessory-enum/@value"/>
        <xsl:variable name="storage-values" select="storage/storage-unit/@value"/>
        <xsl:for-each select="$database/database/accessories/accessory-enum">
          <xsl:if test="$storage-values[string() = current()/@storage] or not(@storage)">
            <xsl:copy>
              <xsl:apply-templates select="@*"/>
              <xsl:if test="$values[string() = current()/@value]">
                <xsl:attribute name="value-is-set">true</xsl:attribute>
              </xsl:if>
            </xsl:copy>
          </xsl:if>
        </xsl:for-each>
      </accessories>
      <peripherals>
        <keyboard>
          <xsl:variable name="value" select="peripherals/keyboard/@value"/>
          <xsl:attribute name="value"><xsl:value-of select="$value"/></xsl:attribute>
          <xsl:for-each select="$database/database/keyboard/keyboard-enum">
            <xsl:copy>
              <xsl:apply-templates select="@*"/>
              <xsl:if test="@value = $value">
                <xsl:attribute name="value-is-set">true</xsl:attribute>
              </xsl:if>
            </xsl:copy>
          </xsl:for-each>
        </keyboard>
        <mouse>
          <xsl:variable name="value" select="peripherals/mouse/@value"/>
          <xsl:attribute name="value"><xsl:value-of select="$value"/></xsl:attribute>
          <xsl:for-each select="$database/database/mouse/mouse-enum">
            <xsl:copy>
              <xsl:apply-templates select="@*"/>
              <xsl:if test="@value = $value">
                <xsl:attribute name="value-is-set">true</xsl:attribute>
              </xsl:if>
            </xsl:copy>
          </xsl:for-each>
        </mouse>
        <screen>
          <xsl:variable name="value" select="peripherals/screen/@value"/>
          <xsl:attribute name="value"><xsl:value-of select="$value"/></xsl:attribute>
          <xsl:for-each select="$database/database/screen/screen-enum">
            <xsl:copy>
              <xsl:apply-templates select="@*"/>
              <xsl:if test="@value = $value">
                <xsl:attribute name="value-is-set">true</xsl:attribute>
              </xsl:if>
            </xsl:copy>
          </xsl:for-each>
        </screen>
      </peripherals>
    </xsl:copy>
  </xsl:template>

  <!-- Replicate unknown elements. -->

  <xsl:template match="@*|node()">
    <xsl:copy>
      <xsl:apply-templates select="@*|node()"/>
    </xsl:copy>
  </xsl:template>

</xsl:stylesheet>
